#include <proto.h>

#include "gtviewport.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtViewPort class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init viewport.
*
\************************************************************************/
GtViewPort *GtMkViewPort(char *n,GtManager *p,GtTag *tgs)
{
	GtViewPort *obj;
	
	if(!(obj=new GtViewPort()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtViewPort object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init viewport.
*
\************************************************************************/
GtViewPort *GtMkViewPort(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtViewPort *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkViewPort(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty viewport.
*
\************************************************************************/
GtViewPort::GtViewPort(void)
{
	maxchild=1;
	cxp=0;
	cyp=0;
}

/************************************************************************\
*
* Set viewport attributes.
*
\************************************************************************/
sbool GtViewPort::Set(GtTag *tgs)
{
	GtTag *t;
	GtWList *wl;
	sbool ch=FALSE;
	
	if(!GtManager::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_ChildXPos:
				cxp=t->d.val;
				ch=TRUE;
				break;
			case GT_ChildYPos:
				cyp=t->d.val;
				ch=TRUE;
				break;
		}
	}
	if(ch && (wl=(GtWList *)GtSFirst(&man)))
		wl->w->SetPosSize(cxp,cyp,cxs,cys);
	return(TRUE);
}

/************************************************************************\
*
* Get viewport attributes.
*
\************************************************************************/
void GtViewPort::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtManager::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_ChildXPos:
				*ptr=cxp;
				break;
			case GT_ChildYPos:
				*ptr=cyp;
				break;
			case GT_ChildXSize:
				*ptr=cxs;
				break;
			case GT_ChildYSize:
				*ptr=cys;
				break;
		}
	}
}
	
/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtViewPort::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtViewPort")==0)
		return(TRUE);
	else
		return(GtManager::IsSubClassOf(cn));
}

/************************************************************************\
*
* Add child object and check for correct type.
*
\************************************************************************/
sbool GtViewPort::AddChild(GtWidget *o,ulong pos)
{
	int xp,yp,xs,ys;
	
	GtManager::AddChild(o,pos);
	if(o->Manage())
	{
		o->GetV(GT_XPos,&xp,GT_YPos,&yp,GT_XSize,&xs,GT_YSize,&ys,GT_DONE);
		cxp=xp;
		cyp=yp;
		cxs=xs;
		cys=ys;
	}
	return(TRUE);
}

/************************************************************************\
*
* Move and/or resize request from child.
*
\************************************************************************/
void GtViewPort::MoveSizeReq(GtWidget *o,word xp,word yp,word xs,word ys)
{
	GtViewPortCD cd;
	
	cxp=xp;
	cyp=yp;
	cxs=xs;
	cys=ys;
	o->SetPosSize(xp,yp,xs,ys);
	cd.cxpos=cxp;
	cd.cypos=cyp;
	cd.cxsize=cxs;
	cd.cysize=cys;
	cd.xsize=xsize;
	cd.ysize=ysize;
	chg.CallCB(this,&cd);
}

/************************************************************************\
*
* Add viewport callback.
*
\************************************************************************/
void GtViewPort::AddCB(ulong tag,GtCallB cb,GtUDPtr ud)
{
	if(tag==GT_ValueChange)
		chg.AddCB(cb,ud);
	else
		GtManager::AddCB(tag,cb,ud);
}

/************************************************************************\
*
* Remove viewport callback.
*
\************************************************************************/
void GtViewPort::RemCB(ulong tag,GtCallB cb)
{
	if(tag==GT_ValueChange)
		chg.RemCB(cb);
	else
		GtManager::RemCB(tag,cb);
}

/************************************************************************\
*
* Change position and size of viewport.
*
\************************************************************************/
void GtViewPort::SetPosSize(word xp,word yp,word xs,word ys)
{
	GtViewPortCD cd;
	word oxs,oys;
	
	oxs=xsize;
	oys=ysize;
	GtManager::SetPosSize(xp,yp,xs,ys);
	if((oxs!=xsize) || (oys!=ysize))
	{
		cd.cxpos=cxp;
		cd.cypos=cyp;
		cd.cxsize=cxs;
		cd.cysize=cys;
		cd.xsize=xsize;
		cd.ysize=ysize;
		chg.CallCB(this,&cd);
	}
}
